# Keys that are expected to be in diagnostics report.
ECC_KEY = 'ECC'
ONBOARDING_KEY = 'onboarding_key'
ETH_MAC_ADDRESS_KEY = 'eth_mac_address'
WIFI_MAC_ADDRESS_KEY = 'wifi_mac_address'
PUBLIC_KEY = 'public_key'
BLUETOOTH_KEY = 'bluetooth'
VARIANT_KEY = 'VARIANT'
FREQUENCY_KEY = 'FREQ'
SERIAL_NUMBER_KEY = 'serial_number'
BALENA_DEVICE_ID_KEY = 'BALENA_DEVICE_UUID'
DISK_IMAGE_KEY = 'disk_image'
LTE_KEY = 'lte'
PF_KEY = 'legacy_pass_fail'
LORA_KEY = 'lora'
